@@ -1,11 +1,79 @@
 package Paws::Kinesis::MemoryCaller;
 use 5.008001;
 
-our $VERSION = "0.02";
+our $VERSION = "0.03";
+
+=head1 NAME
+
+Paws::Kinesis::MemoryCaller - A Paws Caller with in-memory Kinesis.
+
+=head1 SYNOPSIS
+
+    use Paws;
+    use Paws::Kinesis::MemoryCaller;
+
+    my $kinesis = Paws->service('Kinesis',
+        region      => 'N/A',
+        caller      => Paws::Kinesis::MemoryCaller->new(),
+        credentials => Paws::Credential::None->new(),
+    );
+
+    # Create a Kinesis stream...
+    $kinesis->CreateStream(%args);
+
+    # Get a shard iterator...
+    $kinesis->GetShardIterator(%args);
+
+    # Put a record on a stream...
+    $kinesis->PutRecord(%args);
+
+    # Put multiple records on a stream...
+    $kinesis->PutRecords(%args);
+
+    # Get records from a stream...
+    $kinesis->GetRecords(%args);
+
+=head1 DESCRIPTION
+
+Paws::Kinesis::MemoryCaller implements Paws::Net::CallerRole which simulates its
+own streams, shards and records in memory.
+
+The following methods have been implemented:
+
+=over
+
+=item *
+
+CreateStream
+
+=item *
+
+DescribeStream
+
+=item *
+
+GetRecords
+
+=item *
+
+GetShardIterator
+
+=item *
+
+PutRecord
+
+=item *
+
+PutRecords
+
+=back
+
+=cut
 
 use Moose;
 with "Paws::Net::CallerRole";
 
+use namespace::autoclean;
 use Data::UUID;
 use List::AllUtils qw(first_index);
 
@@ -13,8 +81,17 @@ use Paws::Kinesis::DescribeStreamOutput;
 use Paws::Kinesis::GetRecordsOutput;
 use Paws::Kinesis::GetShardIteratorOutput;
 use Paws::Kinesis::PutRecordOutput;
+use Paws::Kinesis::PutRecordsOutput;
 
+use Paws::Kinesis::PutRecord;
+
+use Paws::Kinesis::PutRecordsResultEntry;
+
+use Paws::Kinesis::HashKeyRange;
 use Paws::Kinesis::Record;
+use Paws::Kinesis::Shard;
+use Paws::Kinesis::SequenceNumberRange;
+use Paws::Kinesis::StreamDescription;
 
 has store => (is => 'rw', default => sub { +{} });
 has shard_iterator__address => (is => 'rw', default => sub { +{} });
@@ -28,17 +105,18 @@ sub do_call {
     my $action_class = ref $action;
 
     my $method = {
-        "Paws::Kinesis::CreateStream" => "create_stream",
-        "Paws::Kinesis::DescribeStream" => "describe_stream",
-        "Paws::Kinesis::GetRecords" => "get_records",
-        "Paws::Kinesis::GetShardIterator" => "get_shard_iterator",
-        "Paws::Kinesis::PutRecord" => "put_record",
+        "Paws::Kinesis::CreateStream"       => "_create_stream",
+        "Paws::Kinesis::DescribeStream"     => "_describe_stream",
+        "Paws::Kinesis::GetRecords"         => "_get_records",
+        "Paws::Kinesis::GetShardIterator"   => "_get_shard_iterator",
+        "Paws::Kinesis::PutRecord"          => "_put_record",
+        "Paws::Kinesis::PutRecords"         => "_put_records",
     }->{$action_class} or die "unknown action ($action_class)";
 
     $self->$method($action);
 }
 
-sub create_stream {
+sub _create_stream {
     my $self = shift;
     my ($action) = @_;
 
@@ -51,7 +129,7 @@ sub create_stream {
     return undef;
 }
 
-sub get_shard_iterator {
+sub _get_shard_iterator {
     my $self = shift;
     my ($action) = @_;
 
@@ -129,21 +207,44 @@ sub _create_shard_iterator {
     return $shard_iterator;
 }
 
-sub describe_stream {
+sub _describe_stream {
     my $self = shift;
     my ($action) = @_;
 
     my $stream_name = $action->StreamName;
 
-    return Paws::Kinesis::DescribeStreamOutput->new();
+    my $shard_ids = $self->_get_shard_ids_from_stream_name($stream_name);
 
-    return {
-        name => $stream_name,
-        shard_ids => $self->_get_shard_ids_from_stream_name($stream_name),
-    };
+    my $shards = [
+        map {
+            Paws::Kinesis::Shard->new(
+                HashKeyRange => Paws::Kinesis::HashKeyRange->new(
+                    EndingHashKey   => "",
+                    StartingHashKey => "",
+                ),
+                SequenceNumberRange => Paws::Kinesis::SequenceNumberRange->new(
+                    StartingSequenceNumber => "",
+                ),
+                ShardId => $_,
+            )
+        }
+        @$shard_ids
+    ];
+
+    return Paws::Kinesis::DescribeStreamOutput->new(
+        StreamDescription => Paws::Kinesis::StreamDescription->new(
+            EnhancedMonitoring => [],
+            HasMoreShards => "",
+            RetentionPeriodHours => 24,
+            Shards => $shards,
+            StreamARN => "",
+            StreamName => $stream_name,
+            StreamStatus => "",
+        ),
+    );
 }
 
-sub get_records {
+sub _get_records {
     my $self = shift;
     my ($action) = @_;
 
@@ -173,7 +274,7 @@ sub get_records {
     );
 }
 
-sub put_record {
+sub _put_record {
     my $self = shift;
     my ($action) = @_;
 
@@ -198,6 +299,40 @@ sub put_record {
     );
 }
 
+sub _put_records {
+    my $self = shift;
+    my ($action) = @_;
+
+    my $stream_name = $action->StreamName;
+
+    my $records = [
+        map {
+            my $record = $_;
+
+            my $data = $record->Data;
+            my $paritition_key = $record->PartitionKey;
+
+            my $put_record_output = $self->_put_record(
+                Paws::Kinesis::PutRecord->new(
+                    PartitionKey => $paritition_key,
+                    StreamName   => $stream_name,
+                    Data         => $data,
+                ),
+            );
+
+            Paws::Kinesis::PutRecordsResultEntry->new(
+                ShardId        => $put_record_output->ShardId,
+                SequenceNumber => $put_record_output->SequenceNumber,
+            );
+        }
+        @{$action->Records}
+    ];
+
+    return Paws::Kinesis::PutRecordsOutput->new(
+        Records => $records,
+    );
+}
+
 sub _get_shard_id_from_partition_key {
     my $self = shift;
     my ($action) = @_;
@@ -223,33 +358,7 @@ sub _get_shard_ids_from_stream_name {
     return [ sort { $a <=> $b } keys %$shard_id__records ],
 }
 
-
-1;
-__END__
-
-=encoding utf-8
-
-=head1 NAME
-
-Paws::Kinesis::MemoryCaller - A Paws Caller with in-memory Kinesis.
-
-=head1 SYNOPSIS
-
-    use Paws;
-    use Paws::Kinesis::MemoryCaller;
-
-    my $kinesis = Paws->service('Kinesis',
-        region      => 'N/A',
-        caller      => Paws::Kinesis::MemoryCaller->new(),
-        credentials => Paws::Credential::None->new(),
-    );
-
-    $kinesis->CreateStream(...)
-
-=head1 DESCRIPTION
-
-Paws::Kinesis::MemoryCaller implements Paws::Net::CallerRole which simulates its
-own streams, shards and records.
+__PACKAGE__->meta->make_immutable;
 
 =head1 LICENSE
 
@@ -258,9 +367,16 @@ Copyright (C) Keith Broughton.
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
-=head1 AUTHOR
+=head1 DEVELOPMENT
 
-Keith Broughton E<lt>keithbro256@gmail.comE<gt>
+=head2 Author
+
+Keith Broughton C<< <keithbro [AT] cpan.org> >>
+
+=head2 Bug reports
+
+Please report any bugs or feature requests on GitHub:
+
+L<https://github.com/keithbro/Paws-Kinesis-MemoryCaller/issues>.
 
 =cut
-
